/*******************************************************************************
File Name: CYBLE_custom.h
Version 1.20

Description:
 Contains the function prototypes and constants for the Custom Service.

********************************************************************************
Copyright 2014-2015, Cypress Semiconductor Corporation.  All rights reserved.
You may use this file only in accordance with the license, terms, conditions,
disclaimers, and limitations in the end user license agreement accompanying
the software package with which this file was provided.
*******************************************************************************/


#if !defined(CY_BLE_CYBLE_CUSTOM_H)
#define CY_BLE_CYBLE_CUSTOM_H

#include "BLE_gatt.h"


/***************************************
##Conditional Compilation Parameters
***************************************/

/* Maximum supported Custom Services */
#define CYBLE_CUSTOMS_SERVICE_COUNT                  (0x01u)
#define CYBLE_CUSTOMC_SERVICE_COUNT                  (0x00u)
#define CYBLE_CUSTOM_SERVICE_CHAR_COUNT              (0x07u)
#define CYBLE_CUSTOM_SERVICE_CHAR_DESCRIPTORS_COUNT  (0x02u)

/* Below are the indexes and handles of the defined Custom Services and their characteristics */
#define CYBLE_EBIKEDATA_SERVICE_INDEX   (0x00u) /* Index of eBikeData service in the cyBle_customs array */
#define CYBLE_EBIKEDATA_AMPHOUR_CHAR_INDEX   (0x00u) /* Index of Amphour characteristic */
#define CYBLE_EBIKEDATA_AMPHOUR_CUSTOM_DESCRIPTOR_DESC_INDEX   (0x00u) /* Index of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_AMPHOUR_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_INDEX   (0x01u) /* Index of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_VOLTS_CHAR_INDEX   (0x01u) /* Index of Volts characteristic */
#define CYBLE_EBIKEDATA_VOLTS_CUSTOM_DESCRIPTOR_DESC_INDEX   (0x00u) /* Index of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_VOLTS_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_INDEX   (0x01u) /* Index of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_AMPS_CHAR_INDEX   (0x02u) /* Index of Amps characteristic */
#define CYBLE_EBIKEDATA_AMPS_CUSTOM_DESCRIPTOR_DESC_INDEX   (0x00u) /* Index of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_AMPS_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_INDEX   (0x01u) /* Index of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_SPEED_CHAR_INDEX   (0x03u) /* Index of Speed characteristic */
#define CYBLE_EBIKEDATA_SPEED_CUSTOM_DESCRIPTOR_DESC_INDEX   (0x00u) /* Index of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_SPEED_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_INDEX   (0x01u) /* Index of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_DISTANCE_CHAR_INDEX   (0x04u) /* Index of Distance characteristic */
#define CYBLE_EBIKEDATA_DISTANCE_CUSTOM_DESCRIPTOR_DESC_INDEX   (0x00u) /* Index of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_DISTANCE_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_INDEX   (0x01u) /* Index of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_BATT1_CHAR_INDEX   (0x05u) /* Index of Batt1 characteristic */
#define CYBLE_EBIKEDATA_BATT1_CUSTOM_DESCRIPTOR_DESC_INDEX   (0x00u) /* Index of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_BATT1_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_INDEX   (0x01u) /* Index of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_BATT2_CHAR_INDEX   (0x06u) /* Index of Batt2 characteristic */
#define CYBLE_EBIKEDATA_BATT2_CUSTOM_DESCRIPTOR_DESC_INDEX   (0x00u) /* Index of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_BATT2_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_INDEX   (0x01u) /* Index of Client Characteristic Configuration descriptor */

#define CYBLE_EBIKEDATA_SERVICE_HANDLE   (0x000Cu) /* Handle of eBikeData service */
#define CYBLE_EBIKEDATA_AMPHOUR_CHAR_HANDLE   (0x000Eu) /* Handle of Amphour characteristic */
#define CYBLE_EBIKEDATA_AMPHOUR_CUSTOM_DESCRIPTOR_DESC_HANDLE   (0x000Fu) /* Handle of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_AMPHOUR_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE   (0x0010u) /* Handle of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_VOLTS_CHAR_HANDLE   (0x0012u) /* Handle of Volts characteristic */
#define CYBLE_EBIKEDATA_VOLTS_CUSTOM_DESCRIPTOR_DESC_HANDLE   (0x0013u) /* Handle of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_VOLTS_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE   (0x0014u) /* Handle of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_AMPS_CHAR_HANDLE   (0x0016u) /* Handle of Amps characteristic */
#define CYBLE_EBIKEDATA_AMPS_CUSTOM_DESCRIPTOR_DESC_HANDLE   (0x0017u) /* Handle of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_AMPS_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE   (0x0018u) /* Handle of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_SPEED_CHAR_HANDLE   (0x001Au) /* Handle of Speed characteristic */
#define CYBLE_EBIKEDATA_SPEED_CUSTOM_DESCRIPTOR_DESC_HANDLE   (0x001Bu) /* Handle of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_SPEED_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE   (0x001Cu) /* Handle of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_DISTANCE_CHAR_HANDLE   (0x001Eu) /* Handle of Distance characteristic */
#define CYBLE_EBIKEDATA_DISTANCE_CUSTOM_DESCRIPTOR_DESC_HANDLE   (0x001Fu) /* Handle of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_DISTANCE_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE   (0x0020u) /* Handle of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_BATT1_CHAR_HANDLE   (0x0022u) /* Handle of Batt1 characteristic */
#define CYBLE_EBIKEDATA_BATT1_CUSTOM_DESCRIPTOR_DESC_HANDLE   (0x0023u) /* Handle of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_BATT1_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE   (0x0024u) /* Handle of Client Characteristic Configuration descriptor */
#define CYBLE_EBIKEDATA_BATT2_CHAR_HANDLE   (0x0026u) /* Handle of Batt2 characteristic */
#define CYBLE_EBIKEDATA_BATT2_CUSTOM_DESCRIPTOR_DESC_HANDLE   (0x0027u) /* Handle of Custom Descriptor descriptor */
#define CYBLE_EBIKEDATA_BATT2_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE   (0x0028u) /* Handle of Client Characteristic Configuration descriptor */

	

/***************************************
##Data Struct Definition
***************************************/

#if(0u != CYBLE_CUSTOMS_SERVICE_COUNT)

/* Contains information about Custom Characteristic structure */
typedef struct
{
    /* Custom Characteristic handle */
    CYBLE_GATT_DB_ATTR_HANDLE_T customServiceCharHandle;
    /* Custom Characteristic Descriptors handles */
    CYBLE_GATT_DB_ATTR_HANDLE_T customServiceCharDescriptors[CYBLE_CUSTOM_SERVICE_CHAR_DESCRIPTORS_COUNT];
} CYBLE_CUSTOMS_INFO_T;

/* Structure with Custom Service attribute handles. */
typedef struct
{
    /* Handle of a Custom Service */
    CYBLE_GATT_DB_ATTR_HANDLE_T customServiceHandle;
    
    /* Information about Custom Characteristics */
    CYBLE_CUSTOMS_INFO_T customServiceInfo[CYBLE_CUSTOM_SERVICE_CHAR_COUNT];
} CYBLE_CUSTOMS_T;


#endif /* (0u != CYBLE_CUSTOMS_SERVICE_COUNT) */

/* DOM-IGNORE-BEGIN */
/* The custom Client functionality is not functional in current version of 
* the component.
*/
#if(0u != CYBLE_CUSTOMC_SERVICE_COUNT)

typedef struct
{
    /* Custom Descriptor handle */
    CYBLE_GATT_DB_ATTR_HANDLE_T descHandle;
	/* Custom Descriptor UUID */
	CYBLE_UUID_T uuid;           
    /* UUID Format - 16-bit (0x01) or 128-bit (0x02) */
	uint8 uuidFormat;
   
} CYBLE_CUSTOMC_DESC_T;

typedef struct
{
    /* Characteristic handle */
    CYBLE_GATT_DB_ATTR_HANDLE_T customServiceCharHandle;
	/* Characteristic end handle */
    CYBLE_GATT_DB_ATTR_HANDLE_T customServiceCharEndHandle;
	/* Characteristic UUID */
	CYBLE_UUID_T uuid;           
    /* UUID Format - 16-bit (0x01) or 128-bit (0x02) */
	uint8 uuidFormat;
	/* Number of descriptors */
    uint8 descCount;
    /* Characteristic Descriptors */
    CYBLE_CUSTOMC_DESC_T * customServiceCharDescriptors;
} CYBLE_CUSTOMC_CHAR_T;

/* Structure with discovered attributes information of Custom Service */
typedef struct
{
    /* Custom Service handle */
    CYBLE_GATT_DB_ATTR_HANDLE_T customServiceHandle;
	/* Service UUID */
	CYBLE_UUID_T uuid;           
    /* UUID Format - 16-bit (0x01) or 128-bit (0x02) */
	uint8 uuidFormat;
	/* Number of characteristics */
    uint8 charCount;
    /* Custom Service Characteristics */
    CYBLE_CUSTOMC_CHAR_T * customServiceInfo;
} CYBLE_CUSTOMC_T;

#endif /* (0u != CYBLE_CUSTOMC_SERVICE_COUNT) */
/* DOM-IGNORE-END */


#if(0u != CYBLE_CUSTOMS_SERVICE_COUNT)

extern const CYBLE_CUSTOMS_T cyBle_customs[CYBLE_CUSTOMS_SERVICE_COUNT];

#endif /* (0u != CYBLE_CUSTOMS_SERVICE_COUNT) */

/* DOM-IGNORE-BEGIN */
#if(0u != CYBLE_CUSTOMC_SERVICE_COUNT)

extern CYBLE_CUSTOMC_T cyBle_customc[CYBLE_CUSTOMC_SERVICE_COUNT];

#endif /* (0u != CYBLE_CUSTOMC_SERVICE_COUNT) */
/* DOM-IGNORE-END */

#endif /* CY_BLE_CYBLE_CUSTOM_H  */


/* [] END OF FILE */
