/*******************************************************************************
* File Name: Batt1ADC.h
* Version 2.0
*
* Description:
*  This file contains the function prototypes and constants used in
*  the Sequencing Successive Approximation ADC Component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_ADC_SAR_SEQ_Batt1ADC_H)
#define CY_ADC_SAR_SEQ_Batt1ADC_H

#include "cytypes.h"
#include "CyLib.h"


/***************************************
*      Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
} Batt1ADC_BACKUP_STRUCT;


/**************************************
*    Enumerated Types and Parameters
**************************************/

/*  Clock Source setting constants */
#define Batt1ADC__EXTERNAL 0
#define Batt1ADC__INTERNAL 1

/*  Sample Mode setting constants */
#define Batt1ADC__FREERUNNING 0
#define Batt1ADC__HARDWARESOC 1

/*  Reference type setting constants */
#define Batt1ADC__VDDA_2 0
#define Batt1ADC__VDDA 1
#define Batt1ADC__INTERNAL1024 2
#define Batt1ADC__INTERNAL1024BYPASSED 3
#define Batt1ADC__INTERNALVREF 4
#define Batt1ADC__INTERNALVREFBYPASSED 5
#define Batt1ADC__VDDA_2BYPASSED 6
#define Batt1ADC__EXTERNALVREF 7

/* Input buffer gain setting constants */
#define Batt1ADC__DISABLED 0
#define Batt1ADC__ONE 1
#define Batt1ADC__TWO 2
#define Batt1ADC__FOUR 3
#define Batt1ADC__EIGHT 4
#define Batt1ADC__SIXTEEN 5

/* Negative input setting sonstants in single ended mode */
#define Batt1ADC__VSS 0
#define Batt1ADC__VREF 1
#define Batt1ADC__OTHER 2

/* Compare mode setting constants:
*    Mode0 - Disable
*    Mode1 - Result < Low_Limit
*    Mode2 - Low_Limit <= Result < High_Limit
*    Mode3 - High_Limit <= Result
*    Mode4 - (Result < Low_Limit) or (High_Limit <= Result)
*/
#define Batt1ADC__MODE0 0
#define Batt1ADC__MODE1 1
#define Batt1ADC__MODE2 2
#define Batt1ADC__MODE3 3

#define Batt1ADC__RES8 0
#define Batt1ADC__RES10 1

#define Batt1ADC__RIGHT 0
#define Batt1ADC__LEFT 1

#define Batt1ADC__FSIGNED 1
#define Batt1ADC__FUNSIGNED 0

#define Batt1ADC__ACCUMULATE 0
#define Batt1ADC__FIXEDRESOLUTION 1



/***************************************
*   Conditional Compilation Parameters
****************************************/ 

#ifdef CYIPBLOCK_s8pass4al_VERSION
    #define Batt1ADC_CY_SAR_IP_VER              (CYIPBLOCK_s8pass4al_VERSION)
#else
    #define Batt1ADC_CY_SAR_IP_VER              (0u)
#endif  /* CYIPBLOCK_s8pass4al_VERSION */

#define Batt1ADC_CY_SAR_IP_PSOC4                (0u)
#define Batt1ADC_CY_SAR_IP_BLE                  (1u)


/***************************************
*    Initial Parameter Constants
***************************************/
#define Batt1ADC_DEFAULT_SAMPLE_MODE_SEL        (0u)
#define Batt1ADC_DEFAULT_VREF_SEL               (1u)
#define Batt1ADC_DEFAULT_NEG_INPUT_SEL          (0u)
#define Batt1ADC_DEFAULT_ALT_RESOLUTION_SEL     (0u)
#define Batt1ADC_DEFAULT_JUSTIFICATION_SEL      (0u)
#define Batt1ADC_DEFAULT_DIFF_RESULT_FORMAT_SEL (1u)
#define Batt1ADC_DEFAULT_SE_RESULT_FORMAT_SEL   (1u)
#define Batt1ADC_DEFAULT_CLOCK_SOURCE           (1u)
#define Batt1ADC_DEFAULT_VREF_MV_VALUE          (3300)
#define Batt1ADC_DEFAULT_BUFFER_GAIN            (0u)
#define Batt1ADC_DEFAULT_AVG_SAMPLES_NUM        (0u)
#define Batt1ADC_DEFAULT_AVG_SAMPLES_DIV        (int16)(0x100u >> (7u - 0u))
#define Batt1ADC_DEFAULT_AVG_MODE               (1u)
#define Batt1ADC_MAX_RESOLUTION                 (12u)
#define Batt1ADC_DEFAULT_LOW_LIMIT              (0u)
#define Batt1ADC_DEFAULT_HIGH_LIMIT             (2047u)
#define Batt1ADC_DEFAULT_COMPARE_MODE           (0u)
#define Batt1ADC_DEFAULT_ACLKS_NUM              (4u)
#define Batt1ADC_DEFAULT_BCLKS_NUM              (4u)
#define Batt1ADC_DEFAULT_CCLKS_NUM              (4u)
#define Batt1ADC_DEFAULT_DCLKS_NUM              (4u)
#define Batt1ADC_TOTAL_CHANNELS_NUM             (1u)
#define Batt1ADC_SEQUENCED_CHANNELS_NUM         (1u)
#define Batt1ADC_DEFAULT_EN_CHANNELS            (1u)
#define Batt1ADC_NOMINAL_CLOCK_FREQ             (3000000)
#define Batt1ADC_INJ_CHANNEL_ENABLED            (0u)
#define Batt1ADC_IRQ_REMOVE                     (0u)

/* Determines whether the configuration contains external negative input. */
#define Batt1ADC_SINGLE_PRESENT                 (0u)
#define Batt1ADC_CHANNELS_MODE                  (0u)
#define Batt1ADC_MAX_CHANNELS_EN_MASK           (0xffffu >> (16u - Batt1ADC_SEQUENCED_CHANNELS_NUM))


/***************************************
*        Function Prototypes
***************************************/

void Batt1ADC_Start(void);
void Batt1ADC_Stop(void);
void Batt1ADC_Init(void);
void Batt1ADC_Enable(void);
void Batt1ADC_StartConvert(void);
void Batt1ADC_StopConvert(void);
uint32 Batt1ADC_IsEndConversion(uint32 retMode);
int16 Batt1ADC_GetResult16(uint32 chan);
void Batt1ADC_SetChanMask(uint32 mask);
void Batt1ADC_SetLowLimit(uint32 lowLimit);
void Batt1ADC_SetHighLimit(uint32 highLimit);
void Batt1ADC_SetLimitMask(uint32 mask);
void Batt1ADC_SetSatMask(uint32 mask);
void Batt1ADC_SetOffset(uint32 chan, int16 offset);
void Batt1ADC_SetGain(uint32 chan, int32 adcGain);
#if(Batt1ADC_INJ_CHANNEL_ENABLED)
    void Batt1ADC_EnableInjection(void);
#endif /* Batt1ADC_INJ_CHANNEL_ENABLED */
#if(Batt1ADC_DEFAULT_JUSTIFICATION_SEL == Batt1ADC__RIGHT)
    int16 Batt1ADC_CountsTo_mVolts(uint32 chan, int16 adcCounts);
    int32 Batt1ADC_CountsTo_uVolts(uint32 chan, int16 adcCounts);
    float32 Batt1ADC_CountsTo_Volts(uint32 chan, int16 adcCounts);
#endif /* End Batt1ADC_DEFAULT_JUSTIFICATION_SEL == Batt1ADC__RIGHT */
void Batt1ADC_Sleep(void);
void Batt1ADC_Wakeup(void);
void Batt1ADC_SaveConfig(void);
void Batt1ADC_RestoreConfig(void);

CY_ISR_PROTO( Batt1ADC_ISR );


/**************************************
*           API Constants
**************************************/
/* Constants for Sleep mode states */
#define Batt1ADC_STARTED                    (0x02u)
#define Batt1ADC_ENABLED                    (0x01u)
#define Batt1ADC_DISABLED                   (0x00u)

/*   Constants for IsEndConversion() "retMode" parameter  */
#define Batt1ADC_RETURN_STATUS              (0x01u)
#define Batt1ADC_WAIT_FOR_RESULT            (0x02u)
#define Batt1ADC_RETURN_STATUS_INJ          (0x04u)
#define Batt1ADC_WAIT_FOR_RESULT_INJ        (0x08u)

#define Batt1ADC_MAX_FREQUENCY              (18000000)       /*18Mhz*/

#define Batt1ADC_RESOLUTION_12              (12u)
#define Batt1ADC_RESOLUTION_10              (10u)
#define Batt1ADC_RESOLUTION_8               (8u)

#define Batt1ADC_10US_DELAY                 (10u)

#define Batt1ADC_10V_COUNTS                 (10.0F)
#define Batt1ADC_10MV_COUNTS                (10000)
#define Batt1ADC_10UV_COUNTS                (10000000L)


/***************************************
* Global variables external identifier
***************************************/

extern uint8 Batt1ADC_initVar;
extern volatile int16 Batt1ADC_offset[Batt1ADC_TOTAL_CHANNELS_NUM];
extern volatile int32 Batt1ADC_countsPer10Volt[Batt1ADC_TOTAL_CHANNELS_NUM];


/***************************************
*              Registers
***************************************/

#define Batt1ADC_SAR_CTRL_REG                (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_CTRL )
#define Batt1ADC_SAR_CTRL_PTR                ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_CTRL )

#define Batt1ADC_SAR_SAMPLE_CTRL_REG         (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SAMPLE_CTRL )
#define Batt1ADC_SAR_SAMPLE_CTRL_PTR         ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SAMPLE_CTRL )

#define Batt1ADC_SAR_SAMPLE_TIME01_REG       (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SAMPLE_TIME01 )
#define Batt1ADC_SAR_SAMPLE_TIME01_PTR       ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SAMPLE_TIME01 )

#define Batt1ADC_SAR_SAMPLE_TIME23_REG       (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SAMPLE_TIME23 )
#define Batt1ADC_SAR_SAMPLE_TIME23_PTR       ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SAMPLE_TIME23 )

#define Batt1ADC_SAR_RANGE_THRES_REG         (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_THRES )
#define Batt1ADC_SAR_RANGE_THRES_PTR         ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_THRES )

#define Batt1ADC_SAR_RANGE_COND_REG          (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_COND )
#define Batt1ADC_SAR_RANGE_COND_PTR          ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_COND )

#define Batt1ADC_SAR_CHAN_EN_REG             (*(reg32 *) CYREG_SAR_CHAN_EN )
#define Batt1ADC_SAR_CHAN_EN_PTR             ( (reg32 *) CYREG_SAR_CHAN_EN )

#define Batt1ADC_SAR_START_CTRL_REG          (*(reg32 *) CYREG_SAR_START_CTRL )
#define Batt1ADC_SAR_START_CTRL_PTR          ( (reg32 *) CYREG_SAR_START_CTRL )

#define Batt1ADC_SAR_DFT_CTRL_REG            (*(reg32 *) CYREG_SAR_DFT_CTRL )
#define Batt1ADC_SAR_DFT_CTRL_PTR            ( (reg32 *) CYREG_SAR_DFT_CTRL )

#define Batt1ADC_SAR_CHAN_CONFIG_REG         (*(reg32 *) CYREG_SAR_CHAN_CONFIG00 )
#define Batt1ADC_SAR_CHAN_CONFIG_PTR         ( (reg32 *) CYREG_SAR_CHAN_CONFIG00 )
#define Batt1ADC_SAR_CHAN_CONFIG_IND         CYREG_SAR_CHAN_CONFIG00

#define Batt1ADC_SAR_CHAN_WORK_REG           (*(reg32 *) CYREG_SAR_CHAN_WORK00 )
#define Batt1ADC_SAR_CHAN_WORK_PTR           ( (reg32 *) CYREG_SAR_CHAN_WORK00 )

#define Batt1ADC_SAR_CHAN_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT00 )
#define Batt1ADC_SAR_CHAN_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT00 )
#define Batt1ADC_SAR_CHAN_RESULT_IND         CYREG_SAR_CHAN_RESULT00

#define Batt1ADC_SAR_CHAN0_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT00 )
#define Batt1ADC_SAR_CHAN0_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT00 )

#define Batt1ADC_SAR_CHAN1_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT01 )
#define Batt1ADC_SAR_CHAN1_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT01 )

#define Batt1ADC_SAR_CHAN2_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT02 )
#define Batt1ADC_SAR_CHAN2_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT02 )

#define Batt1ADC_SAR_CHAN3_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT03 )
#define Batt1ADC_SAR_CHAN3_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT03 )

#define Batt1ADC_SAR_CHAN4_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT04 )
#define Batt1ADC_SAR_CHAN4_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT04 )

#define Batt1ADC_SAR_CHAN5_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT05 )
#define Batt1ADC_SAR_CHAN5_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT05 )

#define Batt1ADC_SAR_CHAN6_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT06 )
#define Batt1ADC_SAR_CHAN6_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT06 )

#define Batt1ADC_SAR_CHAN7_RESULT_REG         (*(reg32 *) CYREG_SAR_CHAN_RESULT07 )
#define Batt1ADC_SAR_CHAN7_RESULT_PTR         ( (reg32 *) CYREG_SAR_CHAN_RESULT07 )

#define Batt1ADC_SAR_CHAN_WORK_VALID_REG     (*(reg32 *) CYREG_SAR_CHAN_WORK_VALID)
#define Batt1ADC_SAR_CHAN_WORK_VALID_PTR     ( (reg32 *) CYREG_SAR_CHAN_WORK_VALID)

#define Batt1ADC_SAR_CHAN_RESULT_VALID_REG   ( *(reg32 *) CYREG_SAR_CHAN_RESULT_VALID )
#define Batt1ADC_SAR_CHAN_RESULT_VALID_PTR   ( (reg32 *) CYREG_SAR_CHAN_RESULT_VALID )

#define Batt1ADC_SAR_STATUS_REG              (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_STATUS )
#define Batt1ADC_SAR_STATUS_PTR              ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_STATUS )

#define Batt1ADC_SAR_AVG_START_REG           (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_AVG_STAT )
#define Batt1ADC_SAR_AVG_START_PTR           ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_AVG_STAT )

#define Batt1ADC_SAR_INTR_REG                (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR )
#define Batt1ADC_SAR_INTR_PTR                ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR )

#define Batt1ADC_SAR_INTR_SET_REG            (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_SET )
#define Batt1ADC_SAR_INTR_SET_PTR            ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_SET )

#define Batt1ADC_SAR_INTR_MASK_REG           (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_MASK )
#define Batt1ADC_SAR_INTR_MASK_PTR           ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_MASK )

#define Batt1ADC_SAR_INTR_MASKED_REG         (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_MASKED )
#define Batt1ADC_SAR_INTR_MASKED_PTR         ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_MASKED )

#define Batt1ADC_SAR_SATURATE_INTR_REG       (*(reg32 *) CYREG_SAR_SATURATE_INTR )
#define Batt1ADC_SAR_SATURATE_INTR_PTR       ( (reg32 *) CYREG_SAR_SATURATE_INTR )

#define Batt1ADC_SAR_SATURATE_INTR_SET_REG   (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SATURATE_INTR_SET )
#define Batt1ADC_SAR_SATURATE_INTR_SET_PTR   ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SATURATE_INTR_SET )

#define Batt1ADC_SAR_SATURATE_INTR_MASK_REG  (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SATURATE_INTR_MASK )
#define Batt1ADC_SAR_SATURATE_INTR_MASK_PTR  ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SATURATE_INTR_MASK )

#define Batt1ADC_SAR_SATURATE_INTR_MASKED_REG \
                                                 (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SATURATE_INTR_MASKED )
#define Batt1ADC_SAR_SATURATE_INTR_MASKED_PTR \
                                                 ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_SATURATE_INTR_MASKED )

#define Batt1ADC_SAR_RANGE_INTR_REG          (*(reg32 *) CYREG_SAR_RANGE_INTR )
#define Batt1ADC_SAR_RANGE_INTR_PTR          ( (reg32 *) CYREG_SAR_RANGE_INTR )

#define Batt1ADC_SAR_RANGE_INTR_SET_REG      (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_INTR_SET )
#define Batt1ADC_SAR_RANGE_INTR_SET_PTR      ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_INTR_SET )

#define Batt1ADC_SAR_RANGE_INTR_MASK_REG     (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_INTR_MASK )
#define Batt1ADC_SAR_RANGE_INTR_MASK_PTR     ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_INTR_MASK )

#define Batt1ADC_SAR_RANGE_INTR_MASKED_REG   (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_INTR_MASKED )
#define Batt1ADC_SAR_RANGE_INTR_MASKED_PTR   ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_RANGE_INTR_MASKED )

#define Batt1ADC_SAR_INTR_CAUSE_REG          (*(reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_CAUSE )
#define Batt1ADC_SAR_INTR_CAUSE_PTR          ( (reg32 *) Batt1ADC_cy_psoc4_sar__SAR_INTR_CAUSE )

#define Batt1ADC_SAR_INJ_CHAN_CONFIG_REG     (*(reg32 *) CYREG_SAR_INJ_CHAN_CONFIG)
#define Batt1ADC_SAR_INJ_CHAN_CONFIG_PTR     ( (reg32 *) CYREG_SAR_INJ_CHAN_CONFIG)

#define Batt1ADC_SAR_INJ_RESULT_REG          (*(reg32 *) CYREG_SAR_INJ_RESULT )
#define Batt1ADC_SAR_INJ_RESULT_PTR          ( (reg32 *) CYREG_SAR_INJ_RESULT )

#define Batt1ADC_MUX_SWITCH0_REG             (*(reg32 *)  CYREG_SAR_MUX_SWITCH0 )
#define Batt1ADC_MUX_SWITCH0_PTR             ( (reg32 *)  CYREG_SAR_MUX_SWITCH0 )

#define Batt1ADC_MUX_SWITCH_HW_CTRL_REG      (*(reg32 *)  CYREG_SAR_MUX_SWITCH_HW_CTRL )
#define Batt1ADC_MUX_SWITCH_HW_CTRL_PTR      ( (reg32 *)  CYREG_SAR_MUX_SWITCH_HW_CTRL )

#define Batt1ADC_PUMP_CTRL_REG               (*(reg32 *)  CYREG_SAR_PUMP_CTRL )
#define Batt1ADC_PUMP_CTRL_PTR               ( (reg32 *)  CYREG_SAR_PUMP_CTRL )

#define Batt1ADC_ANA_TRIM_REG                (*(reg32 *)  CYREG_SAR_ANA_TRIM )
#define Batt1ADC_ANA_TRIM_PTR                ( (reg32 *)  CYREG_SAR_ANA_TRIM )

#define Batt1ADC_WOUNDING_REG                (*(reg32 *)  CYREG_SAR_WOUNDING )
#define Batt1ADC_WOUNDING_PTR                ( (reg32 *)  CYREG_SAR_WOUNDING )


/**************************************
*       Register Constants
**************************************/
#define Batt1ADC_INTC_NUMBER                (Batt1ADC_IRQ__INTC_NUMBER)
#define Batt1ADC_INTC_PRIOR_NUMBER          (Batt1ADC_IRQ__INTC_PRIOR_NUM)

/* defines for CTRL register */
#define Batt1ADC_VREF_INTERNAL1024          (0x00000040Lu)
#define Batt1ADC_VREF_EXTERNAL              (0x00000050Lu)
#define Batt1ADC_VREF_VDDA_2                (0x00000060Lu)
#define Batt1ADC_VREF_VDDA                  (0x00000070Lu)
#define Batt1ADC_VREF_INTERNAL1024BYPASSED  (0x000000C0Lu)
#define Batt1ADC_VREF_VDDA_2BYPASSED        (0x000000E0Lu)
#define Batt1ADC_VREF_INTERNALVREF          (0x00000000Lu)
#define Batt1ADC_VREF_INTERNALVREFBYPASSED  (0x00000000Lu)

#define Batt1ADC_NEG_VSSA_KELVIN            (0x00000000Lu)
#define Batt1ADC_NEG_VSSA                   (0x00000200Lu)
#define Batt1ADC_NEG_VREF                   (0x00000E00Lu)
#if(Batt1ADC_TOTAL_CHANNELS_NUM > 1u)
    #define Batt1ADC_NEG_OTHER              (uint16)((uint16)Batt1ADC_cy_psoc4_sarmux_8__VNEG << 9u)
#else
    #define Batt1ADC_NEG_OTHER              (0)
#endif /* Batt1ADC_TOTAL_CHANNELS_NUM > 1u */

#define Batt1ADC_SAR_HW_CTRL_NEGVREF        (0x00002000Lu)

#define Batt1ADC_NORMAL_PWR                 (0x00000000Lu)
#define Batt1ADC_HALF_PWR                   (0x01000000Lu)
#define Batt1ADC_MORE_PWR                   (0x02000000Lu)
#define Batt1ADC_QUARTER_PWR                (0x03000000Lu)

#define Batt1ADC_DSI_SYNC_CONFIG            (0x10000000Lu)
#define Batt1ADC_DSI_MODE                   (0x20000000Lu)
#define Batt1ADC_SWITCH_DISABLE             (0x40000000Lu)
#define Batt1ADC_ENABLE                     (0x80000000Lu)

/* defines for STATUS register */
#define Batt1ADC_STATUS_BUSY                (0x80000000Lu)

/* defines for SAMPLE_CTRL register */
#define Batt1ADC_ALT_RESOLUTION_10BIT       (0x00000001Lu)
#define Batt1ADC_ALT_RESOLUTION_8BIT        (0x00000000Lu)

#define Batt1ADC_DATA_ALIGN_LEFT            (0x00000002Lu)
#define Batt1ADC_DATA_ALIGN_RIGHT           (0x00000000Lu)

#define Batt1ADC_SE_SIGNED_RESULT           (0x00000004Lu)
#define Batt1ADC_SE_UNSIGNED_RESULT         (0x00000000Lu)

#define Batt1ADC_DIFF_SIGNED_RESULT         (0x00000008Lu)
#define Batt1ADC_DIFF_UNSIGNED_RESULT       (0x00000000Lu)

#define Batt1ADC_AVG_CNT_OFFSET             (4u)
#define Batt1ADC_AVG_CNT_MASK               (0x00000070Lu)
#define Batt1ADC_AVG_SHIFT                  (0x00000080Lu)

#define Batt1ADC_CONTINUOUS_EN              (0x00010000Lu)
#define Batt1ADC_DSI_TRIGGER_EN             (0x00020000Lu)
#define Batt1ADC_DSI_TRIGGER_LEVEL          (0x00040000Lu)
#define Batt1ADC_DSI_SYNC_TRIGGER           (0x00080000Lu)
#define Batt1ADC_EOS_DSI_OUT_EN             (0x80000000Lu)

/* defines for SAMPLE_TIME01 / SAMPLE_TIME23 registers */
#define Batt1ADC_SAMPLE_TIME13_OFFSET       (16u)
#define Batt1ADC_SAMPLE_TIME02_MASK         (0x000003FFLu)
#define Batt1ADC_SAMPLE_TIME13_MASK         (0x03FF0000Lu)

/* defines for RANGE_THRES registers */
#define Batt1ADC_RANGE_HIGH_OFFSET          (16u)
#define Batt1ADC_RANGE_HIGH_MASK            (0xFFFF0000Lu)
#define Batt1ADC_RANGE_LOW_MASK             (0x0000FFFFLu)

/* defines for RANGE_COND register */
/* Compare mode setting constants:
*    BELOW   - Result < Low_Limit
*    INSIDE  - Low_Limit <= Result < High_Limit
*    ABOVE   - High_Limit <= Result
*    OUTSIDE - (Result < Low_Limit) or (High_Limit <= Result)
*/
#define Batt1ADC_CMP_MODE_BELOW             (0x00000000Lu)
#define Batt1ADC_CMP_MODE_INSIDE            (0x40000000Lu)
#define Batt1ADC_CMP_MODE_ABOVE             (0x80000000Lu)
#define Batt1ADC_CMP_MODE_OUTSIDE           (0xC0000000Lu)
#define Batt1ADC_CMP_OFFSET                 (30u)

/* defines for _START_CTRL register */
#define Batt1ADC_FW_TRIGGER                 (0x00000001Lu)

/* defines for DFT_CTRL register */
#define Batt1ADC_DLY_INC                    (0x00000001Lu)
#define Batt1ADC_HIZ                        (0x00000002Lu)
#define Batt1ADC_DFT_INC_MASK               (0x000F0000Lu)
#define Batt1ADC_DFT_OUTC_MASK              (0x00700000Lu)
#define Batt1ADC_SEL_CSEL_DFT_MASK          (0x0F000000Lu)

/* configuration for clock speed > 9 Mhz based on
* characterization results
*/
#define Batt1ADC_SEL_CSEL_DFT_CHAR          (0x03000000Lu)
#define Batt1ADC_EN_CSEL_DFT                (0x10000000Lu)
#define Batt1ADC_DCEN                       (0x20000000Lu)
#define Batt1ADC_ADFT_OVERRIDE              (0x80000000Lu)

/* defines for CHAN_CONFIG / DIE_CHAN_CONFIG register
*  and channelsConfig parameter
*/
#define Batt1ADC_SARMUX_VIRT_SELECT         (0x00000070Lu)
#define Batt1ADC_DIFFERENTIAL_EN            (0x00000100Lu)
#define Batt1ADC_ALT_RESOLUTION_ON          (0x00000200Lu)
#define Batt1ADC_AVERAGING_EN               (0x00000400Lu)

#define Batt1ADC_SAMPLE_TIME_SEL_SHIFT      (12u)
#define Batt1ADC_SAMPLE_TIME_SEL_MASK       (0x00003000Lu)

#define Batt1ADC_CHANNEL_CONFIG_MASK        (0x00003700Lu)

/* for CHAN_CONFIG only */
#define Batt1ADC_DSI_OUT_EN                 (0x80000000Lu)

/* for INJ_CHAN_CONFIG only */
#define Batt1ADC_INJ_TAILGATING             (0x40000000Lu)
#define Batt1ADC_INJ_CHAN_EN                (0x80000000Lu)

/* defines for CHAN_WORK register */
#define Batt1ADC_SAR_WRK_MAX_12BIT          (0x00001000Lu)
#define Batt1ADC_SAR_WRK_MAX_10BIT          (0x00000400Lu)
#define Batt1ADC_SAR_WRK_MAX_8BIT           (0x00000100Lu)

/* defines for CHAN_RESULT register */
#define Batt1ADC_RESULT_MASK                (0x0000FFFFLu)
#define Batt1ADC_SATURATE_INTR_MIR          (0x20000000Lu)
#define Batt1ADC_RANGE_INTR_MIR             (0x40000000Lu)
#define Batt1ADC_CHAN_RESULT_VALID_MIR      (0x80000000Lu)

/* defines for INTR_MASK register */
#define Batt1ADC_EOS_MASK                   (0x00000001Lu)
#define Batt1ADC_OVERFLOW_MASK              (0x00000002Lu)
#define Batt1ADC_FW_COLLISION_MASK          (0x00000004Lu)
#define Batt1ADC_DSI_COLLISION_MASK         (0x00000008Lu)
#define Batt1ADC_INJ_EOC_MASK               (0x00000010Lu)
#define Batt1ADC_INJ_SATURATE_MASK          (0x00000020Lu)
#define Batt1ADC_INJ_RANGE_MASK             (0x00000040Lu)
#define Batt1ADC_INJ_COLLISION_MASK         (0x00000080Lu)

/* defines for INJ_RESULT register */
#define Batt1ADC_INJ_COLLISION_INTR_MIR     (0x10000000Lu)
#define Batt1ADC_INJ_SATURATE_INTR_MIR      (0x20000000Lu)
#define Batt1ADC_INJ_RANGE_INTR_MIR         (0x40000000Lu)
#define Batt1ADC_INJ_EOC_INTR_MIR           (0x80000000Lu)

/* defines for MUX_SWITCH0 register */
#define Batt1ADC_MUX_FW_VSSA_VMINUS         (0x00010000Lu)

/* defines for PUMP_CTRL register */
#define Batt1ADC_PUMP_CTRL_ENABLED          (0x80000000Lu)

/* additional defines for channelsConfig parameter */
#define Batt1ADC_IS_SATURATE_EN_MASK        (0x00000001Lu)
#define Batt1ADC_IS_RANGE_CTRL_EN_MASK      (0x00000002Lu)

/* defines for WOUNDING register */
#define Batt1ADC_WOUNDING_12BIT             (0x00000000Lu)
#define Batt1ADC_WOUNDING_10BIT             (0x00000001Lu)
#define Batt1ADC_WOUNDING_8BIT              (0x00000002Lu)

/* Trim value based on characterization */
#define Batt1ADC_TRIM_COEF                  (2u)

#if(Batt1ADC_MAX_RESOLUTION == Batt1ADC_RESOLUTION_10)
    #define Batt1ADC_ALT_WOUNDING           Batt1ADC_WOUNDING_10BIT
#else
    #define Batt1ADC_ALT_WOUNDING           Batt1ADC_WOUNDING_8BIT
#endif /* Batt1ADC_MAX_RESOLUTION == Batt1ADC_RESOLUTION_10 */

#if(Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__VDDA_2)
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_VDDA_2
#elif(Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__VDDA)
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_VDDA
#elif(Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__INTERNAL1024)
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_INTERNAL1024
#elif(Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__INTERNAL1024BYPASSED)
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_INTERNAL1024BYPASSED
#elif(Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__INTERNALVREF)
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_INTERNALVREF
#elif(Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__INTERNALVREFBYPASSED)
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_INTERNALVREFBYPASSED
#elif(Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__VDDA_2BYPASSED)
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_VDDA_2BYPASSED
#else
    #define Batt1ADC_DEFAULT_VREF_SOURCE    Batt1ADC_VREF_EXTERNAL
#endif /* Batt1ADC_DEFAULT_VREF_SEL == Batt1ADC__VDDA_2 */

#if(Batt1ADC_DEFAULT_NEG_INPUT_SEL == Batt1ADC__VSS)
    /* Connect NEG input of SARADC to VSSA close to the SARADC for single channel mode */
    #if(Batt1ADC_TOTAL_CHANNELS_NUM == 1u)
        #define Batt1ADC_DEFAULT_SE_NEG_INPUT    Batt1ADC_NEG_VSSA
    #else
        #define Batt1ADC_DEFAULT_SE_NEG_INPUT    Batt1ADC_NEG_VSSA_KELVIN
    #endif /* (Batt1ADC_TOTAL_CHANNELS_NUM == 1u) */
    /* Do not connect VSSA to VMINUS when one channel in differential mode used */
    #if((Batt1ADC_TOTAL_CHANNELS_NUM == 1u) && (Batt1ADC_CHANNELS_MODE != 0u))
        #define Batt1ADC_DEFAULT_MUX_SWITCH0     0u
    #else    /* miltiple channels or one single channel */
        #define Batt1ADC_DEFAULT_MUX_SWITCH0     Batt1ADC_MUX_FW_VSSA_VMINUS
    #endif /* (Batt1ADC_TOTAL_CHANNELS_NUM == 1u) */
#elif(Batt1ADC_DEFAULT_NEG_INPUT_SEL == Batt1ADC__VREF)
    /* Do not connect VNEG to VREF when one channel in differential mode used */
    #if((Batt1ADC_TOTAL_CHANNELS_NUM == 1u) && (Batt1ADC_CHANNELS_MODE != 0u))
        #define Batt1ADC_DEFAULT_SE_NEG_INPUT    0u
    #else    /* miltiple channels or one single channel */
        #define Batt1ADC_DEFAULT_SE_NEG_INPUT    Batt1ADC_NEG_VREF
    #endif /* (Batt1ADC_TOTAL_CHANNELS_NUM == 1u) */
    #define Batt1ADC_DEFAULT_MUX_SWITCH0     0u
#elif (Batt1ADC_SINGLE_PRESENT != 0u)
    #define Batt1ADC_DEFAULT_SE_NEG_INPUT    Batt1ADC_NEG_OTHER
    #define Batt1ADC_DEFAULT_MUX_SWITCH0     0u
#else
    #define Batt1ADC_DEFAULT_SE_NEG_INPUT    0u
    #define Batt1ADC_DEFAULT_MUX_SWITCH0     0u
#endif /* Batt1ADC_DEFAULT_NEG_INPUT_SEL == Batt1ADC__VREF */

/* If the SAR is configured for multiple channels, always set SAR_HW_CTRL_NEGVREF to 1 */
#if(Batt1ADC_TOTAL_CHANNELS_NUM == 1u)
    #define Batt1ADC_DEFAULT_HW_CTRL_NEGVREF 0u
#else
    #define Batt1ADC_DEFAULT_HW_CTRL_NEGVREF Batt1ADC_SAR_HW_CTRL_NEGVREF
#endif /* (Batt1ADC_TOTAL_CHANNELS_NUM == 1u) */


#if(Batt1ADC_DEFAULT_ALT_RESOLUTION_SEL == Batt1ADC__RES8)
    #define Batt1ADC_DEFAULT_ALT_RESOLUTION     (Batt1ADC_ALT_RESOLUTION_8BIT)
    #define Batt1ADC_DEFAULT_MAX_WRK_ALT        (Batt1ADC_SAR_WRK_MAX_8BIT)
#else
    #define Batt1ADC_DEFAULT_ALT_RESOLUTION     (Batt1ADC_ALT_RESOLUTION_10BIT)
    #define Batt1ADC_DEFAULT_MAX_WRK_ALT        (Batt1ADC_SAR_WRK_MAX_10BIT)
#endif /* End Batt1ADC_DEFAULT_ALT_RESOLUTION_SEL == Batt1ADC__RES8 */

#if(Batt1ADC_DEFAULT_JUSTIFICATION_SEL == Batt1ADC__RIGHT)
    #define Batt1ADC_DEFAULT_JUSTIFICATION  Batt1ADC_DATA_ALIGN_RIGHT
#else
    #define Batt1ADC_DEFAULT_JUSTIFICATION  Batt1ADC_DATA_ALIGN_LEFT
#endif /* Batt1ADC_DEFAULT_JUSTIFICATION_SEL == Batt1ADC__RIGHT */

#if(Batt1ADC_DEFAULT_DIFF_RESULT_FORMAT_SEL == Batt1ADC__FSIGNED)
    #define Batt1ADC_DEFAULT_DIFF_RESULT_FORMAT Batt1ADC_DIFF_SIGNED_RESULT
#else
    #define Batt1ADC_DEFAULT_DIFF_RESULT_FORMAT Batt1ADC_DIFF_UNSIGNED_RESULT
#endif /* Batt1ADC_DEFAULT_DIFF_RESULT_FORMAT_SEL == Batt1ADC__FSIGNED */

#if(Batt1ADC_DEFAULT_SE_RESULT_FORMAT_SEL == Batt1ADC__FSIGNED)
    #define Batt1ADC_DEFAULT_SE_RESULT_FORMAT Batt1ADC_SE_SIGNED_RESULT
#else
    #define Batt1ADC_DEFAULT_SE_RESULT_FORMAT Batt1ADC_SE_UNSIGNED_RESULT
#endif /* Batt1ADC_DEFAULT_SE_RESULT_FORMAT_SEL == Batt1ADC__FSIGNED */

#if(Batt1ADC_DEFAULT_SAMPLE_MODE_SEL == Batt1ADC__FREERUNNING)
    #define Batt1ADC_DSI_TRIGGER        0u
#else /* Firmware trigger */
    #define Batt1ADC_DSI_TRIGGER        (Batt1ADC_DSI_TRIGGER_EN | Batt1ADC_DSI_SYNC_TRIGGER)
#endif /* End Batt1ADC_DEFAULT_SAMPLE_MODE == Batt1ADC__FREERUNNING */

#if(Batt1ADC_INJ_CHANNEL_ENABLED)
    #define Batt1ADC_SAR_INTR_MASK      (Batt1ADC_EOS_MASK | Batt1ADC_INJ_EOC_MASK)
#else
    #define Batt1ADC_SAR_INTR_MASK      (Batt1ADC_EOS_MASK)
#endif /* Batt1ADC_INJ_CHANNEL_ENABLED*/

#if(Batt1ADC_DEFAULT_AVG_MODE == Batt1ADC__FIXEDRESOLUTION)
    #define Batt1ADC_AVG_SHIFT_MODE     Batt1ADC_AVG_SHIFT
#else
    #define Batt1ADC_AVG_SHIFT_MODE     0u
#endif /* End Batt1ADC_DEFAULT_AVG_MODE */

#define Batt1ADC_COMPARE_MODE           (uint32)((uint32)(Batt1ADC_DEFAULT_COMPARE_MODE) \
                                                << Batt1ADC_CMP_OFFSET)

#if(Batt1ADC_TOTAL_CHANNELS_NUM > 1u)
    #define Batt1ADC_DEFAULT_SWITCH_CONF    0u
#else /* Disable SAR sequencer from enabling routing switches in single channel mode */
    #define Batt1ADC_DEFAULT_SWITCH_CONF    Batt1ADC_SWITCH_DISABLE
#endif /* End Batt1ADC_TOTAL_CHANNELS_NUM > 1 */

#define Batt1ADC_DEFAULT_POWER \
       ((Batt1ADC_NOMINAL_CLOCK_FREQ > (Batt1ADC_MAX_FREQUENCY / 4)) ? Batt1ADC_NORMAL_PWR : \
       ((Batt1ADC_NOMINAL_CLOCK_FREQ > (Batt1ADC_MAX_FREQUENCY / 8)) ? Batt1ADC_HALF_PWR : \
                                                                                       Batt1ADC_QUARTER_PWR))

#define Batt1ADC_DEFAULT_CTRL_REG_CFG       (Batt1ADC_DEFAULT_VREF_SOURCE \
                                                   | Batt1ADC_DEFAULT_SE_NEG_INPUT \
                                                   | Batt1ADC_DEFAULT_HW_CTRL_NEGVREF \
                                                   | Batt1ADC_DEFAULT_POWER \
                                                   | Batt1ADC_DSI_SYNC_CONFIG \
                                                   | Batt1ADC_DEFAULT_SWITCH_CONF)

#define Batt1ADC_DEFAULT_SAMPLE_CTRL_REG_CFG (Batt1ADC_DEFAULT_DIFF_RESULT_FORMAT \
                                                    | Batt1ADC_DEFAULT_SE_RESULT_FORMAT \
                                                    | Batt1ADC_DEFAULT_JUSTIFICATION \
                                                    | Batt1ADC_DEFAULT_ALT_RESOLUTION \
                                           | (uint8)(Batt1ADC_DEFAULT_AVG_SAMPLES_NUM \
                                                   << Batt1ADC_AVG_CNT_OFFSET) \
                                                    | Batt1ADC_AVG_SHIFT_MODE \
                                                    | Batt1ADC_DSI_TRIGGER \
                                                    | Batt1ADC_EOS_DSI_OUT_EN)

#define Batt1ADC_DEFAULT_RANGE_THRES_REG_CFG (Batt1ADC_DEFAULT_LOW_LIMIT \
            | (uint32)((uint32)Batt1ADC_DEFAULT_HIGH_LIMIT << Batt1ADC_RANGE_HIGH_OFFSET))

#define Batt1ADC_DEFAULT_SAMPLE_TIME01_REG_CFG (Batt1ADC_DEFAULT_ACLKS_NUM \
            | (uint32)((uint32)Batt1ADC_DEFAULT_BCLKS_NUM << Batt1ADC_SAMPLE_TIME13_OFFSET))

#define Batt1ADC_DEFAULT_SAMPLE_TIME23_REG_CFG (Batt1ADC_DEFAULT_CCLKS_NUM \
            | (uint32)((uint32)Batt1ADC_DEFAULT_DCLKS_NUM << Batt1ADC_SAMPLE_TIME13_OFFSET))


#endif /* End CY_ADC_SAR_SEQ_Batt1ADC_H */


/* [] END OF FILE */
