/*******************************************************************************
* File Name: Batt1ADC_PM.c
* Version 2.0
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Batt1ADC.h"


/***************************************
* Local data allocation
***************************************/

static Batt1ADC_BACKUP_STRUCT  Batt1ADC_backup =
{
    Batt1ADC_DISABLED
};


/*******************************************************************************
* Function Name: Batt1ADC_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Batt1ADC_SaveConfig(void)
{
    /* All configuration registers are marked as [reset_all_retention] */
}


/*******************************************************************************
* Function Name: Batt1ADC_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void Batt1ADC_RestoreConfig(void)
{
    /* All congiguration registers are marked as [reset_all_retention] */
}


/*******************************************************************************
* Function Name: Batt1ADC_Sleep
********************************************************************************
*
* Summary:
*  Stops the ADC operation and saves the configuration registers and component
*  enable state. Should be called just prior to entering sleep.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  Batt1ADC_backup - modified.
*
*******************************************************************************/
void Batt1ADC_Sleep(void)
{
    /* During deepsleep/ hibernate mode keep SARMUX active, i.e. do not open
    *   all switches (disconnect), to be used for ADFT
    */
    Batt1ADC_SAR_DFT_CTRL_REG |= Batt1ADC_ADFT_OVERRIDE;
    if((Batt1ADC_SAR_CTRL_REG  & Batt1ADC_ENABLE) != 0u)
    {
        if((Batt1ADC_SAR_SAMPLE_CTRL_REG & Batt1ADC_CONTINUOUS_EN) != 0u)
        {
            Batt1ADC_backup.enableState = Batt1ADC_ENABLED | Batt1ADC_STARTED;
        }
        else
        {
            Batt1ADC_backup.enableState = Batt1ADC_ENABLED;
        }
        Batt1ADC_StopConvert();
        Batt1ADC_Stop();
    }
    else
    {
        Batt1ADC_backup.enableState = Batt1ADC_DISABLED;
    }
}


/*******************************************************************************
* Function Name: Batt1ADC_Wakeup
********************************************************************************
*
* Summary:
*  Restores the component enable state and configuration registers.
*  This should be called just after awaking from sleep mode.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  Batt1ADC_backup - used.
*
*******************************************************************************/
void Batt1ADC_Wakeup(void)
{
    Batt1ADC_SAR_DFT_CTRL_REG &= (uint32)~Batt1ADC_ADFT_OVERRIDE;
    if(Batt1ADC_backup.enableState != Batt1ADC_DISABLED)
    {
        Batt1ADC_Enable();
        if((Batt1ADC_backup.enableState & Batt1ADC_STARTED) != 0u)
        {
            Batt1ADC_StartConvert();
        }
    }
}
/* [] END OF FILE */
