/*******************************************************************************
* File Name: Batt1Mon.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Batt1Mon.h"

#define SetP4PinDriveMode(shift, mode)  \
    do { \
        Batt1Mon_PC =   (Batt1Mon_PC & \
                                (uint32)(~(uint32)(Batt1Mon_DRIVE_MODE_IND_MASK << (Batt1Mon_DRIVE_MODE_BITS * (shift))))) | \
                                (uint32)((uint32)(mode) << (Batt1Mon_DRIVE_MODE_BITS * (shift))); \
    } while (0)


/*******************************************************************************
* Function Name: Batt1Mon_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None 
*  
*******************************************************************************/
void Batt1Mon_Write(uint8 value) 
{
    uint8 drVal = (uint8)(Batt1Mon_DR & (uint8)(~Batt1Mon_MASK));
    drVal = (drVal | ((uint8)(value << Batt1Mon_SHIFT) & Batt1Mon_MASK));
    Batt1Mon_DR = (uint32)drVal;
}


/*******************************************************************************
* Function Name: Batt1Mon_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  Batt1Mon_DM_STRONG     Strong Drive 
*  Batt1Mon_DM_OD_HI      Open Drain, Drives High 
*  Batt1Mon_DM_OD_LO      Open Drain, Drives Low 
*  Batt1Mon_DM_RES_UP     Resistive Pull Up 
*  Batt1Mon_DM_RES_DWN    Resistive Pull Down 
*  Batt1Mon_DM_RES_UPDWN  Resistive Pull Up/Down 
*  Batt1Mon_DM_DIG_HIZ    High Impedance Digital 
*  Batt1Mon_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void Batt1Mon_SetDriveMode(uint8 mode) 
{
	SetP4PinDriveMode(Batt1Mon__0__SHIFT, mode);
}


/*******************************************************************************
* Function Name: Batt1Mon_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro Batt1Mon_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 Batt1Mon_Read(void) 
{
    return (uint8)((Batt1Mon_PS & Batt1Mon_MASK) >> Batt1Mon_SHIFT);
}


/*******************************************************************************
* Function Name: Batt1Mon_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 Batt1Mon_ReadDataReg(void) 
{
    return (uint8)((Batt1Mon_DR & Batt1Mon_MASK) >> Batt1Mon_SHIFT);
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(Batt1Mon_INTSTAT) 

    /*******************************************************************************
    * Function Name: Batt1Mon_ClearInterrupt
    ********************************************************************************
    *
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 Batt1Mon_ClearInterrupt(void) 
    {
		uint8 maskedStatus = (uint8)(Batt1Mon_INTSTAT & Batt1Mon_MASK);
		Batt1Mon_INTSTAT = maskedStatus;
        return maskedStatus >> Batt1Mon_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 


/* [] END OF FILE */
